@@ -1,5 +1,5 @@
 /*
- * $Id: di.h,v 1.44 2010-03-18 01:17:49-07 bll Exp $
+ * $Id: di.h,v 1.52 2010-07-19 08:24:41-07 bll Exp $
  * $Source: /home/bll/DI/di/RCS/di.h,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
@@ -90,16 +90,6 @@
 # endif
 #endif
 
-#if ! _dcl_errno
-  extern int     errno;
-#endif
-#if ! _dcl_optind
-  extern int optind;
-#endif
-#if ! _dcl_optarg
-  extern char *optarg;
-#endif
-
 #define DI_NAME_LEN            MAXPATHLEN
 #define DI_SPEC_NAME_LEN       MAXPATHLEN
 #define DI_OPT_LEN             MAXPATHLEN
@@ -158,6 +148,7 @@ typedef struct
                                                  /* local?               */
     char            isReadOnly;                  /* is this mount point  */
                                                  /* read-only?           */
+    char            isLoopback;                  /* lofs or none fs type? */
     char            name [DI_NAME_LEN + 1];         /* mount point          */
     char            special [DI_SPEC_NAME_LEN + 1]; /* special device name  */
     char            fsType [DI_TYPE_LEN + 1];       /* type of file system  */
@@ -165,14 +156,45 @@ typedef struct
     char            mountTime [DI_MNT_TIME_LEN + 1];
 } diDiskInfo_t;
 
-# if defined(__cplusplus)
+typedef struct
+{
+    char            *special;
+    char            *name;
+    char            *type;
+    Uid_t           uid;
+    Gid_t           gid;
+    _fs_size_t      blockSize;
+    _fs_size_t      limit;
+    _fs_size_t      used;
+    _fs_size_t      ilimit;
+    _fs_size_t      iused;
+} diQuota_t;
+
+# if defined (__cplusplus) || defined (c_plusplus)
    extern "C" {
 # endif
 
+#if ! _dcl_errno
+  extern int errno;
+#endif
+#if ! _dcl_optind
+  extern int optind;
+#endif
+#if ! _dcl_optarg
+  extern char *optarg;
+#endif
+
+/* workaround for AIX - mntctl not declared */
+# if _lib_mntctl && _npt_mntctl
+  extern int mntctl _((int, int, char *));
+# endif
+
  /* digetentries.c */
 extern int  di_getDiskEntries       _((diDiskInfo_t **, int *));
  /* digetinfo.c */
 extern void di_getDiskInfo          _((diDiskInfo_t **, int *));
+ /* diquota.c */
+extern void diquota                 _((diQuota_t *));
  /* getopt.c */
 # if ! _lib_getopt || _npt_getopt
 extern int getopt                   _((int, char * const [], const char *));
@@ -188,7 +210,7 @@ extern char *strstr                 _((const char *, const char *));
  /* trimchar.c */
 extern void trimChar                _((char *, int));
  /* realloc.c */
-extern void *_realloc               _((void *, Size_t));
+extern _pvoid _realloc               _((_pvoid, Size_t));
  /* didiskutil.c */
 extern void di_initDiskInfo _((diDiskInfo_t *));
 extern void di_saveBlockSizes _((diDiskInfo_t *, _fs_size_t, _fs_size_t, _fs_size_t, _fs_size_t));
@@ -197,21 +219,15 @@ extern void di_saveInodeSizes _((diDiskInfo_t *, _fs_size_t, _fs_size_t, _fs_siz
     ! _lib_getmntinfo && \
     ! _lib_getfsstat && \
     ! _lib_getvfsstat && \
-	! _lib_mntctl && \
-	! _class_os__Volumes
+    ! _lib_mntctl && \
+    ! _class_os__Volumes
 extern char *chkMountOptions        _((const char *, const char *));
 #endif
 extern void convertMountOptions     _((long, diDiskInfo_t *));
 extern void convertNFSMountOptions  _((long, long, long, diDiskInfo_t *));
 extern void di_testRemoteDisk       _((diDiskInfo_t *));
 
-/* workaround for cygwin                                              */
-/* if we have a getopt header, there's probably a getopt lib function */
-# if ! _lib_getopt && ! _hdr_getopt
-extern int getopt _((int, char * const [], const char *));
-# endif
-
-# if defined(__cplusplus)
+# if defined (__cplusplus) || defined (c_plusplus)
    }
 # endif
 
@@ -235,21 +251,21 @@ typedef struct {
   void    *retlen;         /* Return length address */
 } VMS_ITMLST;
 
-#define DVI_IS_STRING 1
-#define DVI_IS_LONGWORD 2
-#define DVI_IS_QUADWORD 3
-#define DVI_IS_WORD 4
-#define DVI_IS_BYTE 5
-#define DVI_IS_VMSDATE 6
-#define DVI_IS_BITMAP 7   /* Each bit in the return value indicates something */
-#define DVI_IS_ENUM 8     /* Each returned value has a name, and we ought to */
+# define DVI_IS_STRING 1
+# define DVI_IS_LONGWORD 2
+# define DVI_IS_QUADWORD 3
+# define DVI_IS_WORD 4
+# define DVI_IS_BYTE 5
+# define DVI_IS_VMSDATE 6
+# define DVI_IS_BITMAP 7  /* Each bit in the return value indicates something */
+# define DVI_IS_ENUM 8    /* Each returned value has a name, and we ought to */
                           /* return the name instead of the value */
-#define DVI_IS_ODD 9      /* A catchall */
+# define DVI_IS_ODD 9     /* A catchall */
 
 
-#define DVI_IS_INPUT (1<<0)
-#define DVI_IS_OUTPUT (1<<1)
-#define DVI_ENT(a, b, c) {#a, DVI$_##a, b, c, DVI_IS_OUTPUT}
+# define DVI_IS_INPUT (1<<0)
+# define DVI_IS_OUTPUT (1<<1)
+# define DVI_ENT(a, b, c) {#a, DVI$_##a, b, c, DVI_IS_OUTPUT}
 
 typedef struct {
   char *name;         /* Pointer to the item name */
@@ -262,7 +278,7 @@ typedef struct {
 } genericID_t ;
 
 /* Macro to fill in a 'traditional' item-list entry */
-#define init_itemlist(ile, length, code, bufaddr, retlen_addr) \
+# define init_itemlist(ile, length, code, bufaddr, retlen_addr) \
 { \
     (ile)->buflen = (length); \
     (ile)->itmcode = (code); \
