@@ -1,5 +1,5 @@
 /*
- * $Id: didiskutil.c,v 1.6 2010-03-12 00:25:15-08 bll Exp $
+ * $Id: didiskutil.c,v 1.10 2010-07-14 01:54:32-07 bll Exp $
  * $Source: /home/bll/DI/di/RCS/didiskutil.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
@@ -8,7 +8,9 @@
 #include "di.h"
 #include "dimntopt.h"
 
-#include <stdio.h>
+#if _hdr_stdio
+# include <stdio.h>
+#endif
 #if _hdr_stdlib
 # include <stdlib.h>
 #endif
@@ -64,6 +66,7 @@ di_initDiskInfo (diptr)
     diptr->printFlag = DI_PRNT_OK;
     diptr->isLocal = TRUE;
     diptr->isReadOnly = FALSE;
+    diptr->isLoopback = FALSE;
 }
 
 void
