@@ -1,5 +1,5 @@
 /*
- * $Id: digetentries.c,v 1.15 2010-03-20 13:42:22-07 bll Exp $
+ * $Id: digetentries.c,v 1.29 2010-07-14 01:54:36-07 bll Exp $
  * $Source: /home/bll/DI/di/RCS/digetentries.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
@@ -18,16 +18,30 @@
 #include "di.h"
 #include "dimntopt.h"
 
-#include <stdio.h>
+#if _hdr_stdio
+# include <stdio.h>
+#endif
 #if _hdr_stdlib
 # include <stdlib.h>
 #endif
+#if _hdr_dirent
+# include <dirent.h>
+#endif
 #if _sys_types
 # include <sys/types.h>
 #endif
 #if _sys_param
 # include <sys/param.h>
 #endif
+#if _sys_ftype                      /* QNX */
+# include <sys/ftype.h>
+#endif
+#if _sys_dcmd_blk                   /* QNX */
+# include <sys/dcmd_blk.h>
+#endif
+#if _sys_io                         /* QNX */
+# include <sys/io.h>
+#endif
 #if _hdr_errno
 # include <errno.h>
 #endif
@@ -46,7 +60,7 @@
 #if _hdr_time
 # include <time.h>
 #endif
-#if _sys_time && ((! _hdr_time) || (_include_time))
+#if _sys_time && _inc_conflict__hdr_time__sys_time
 # include <sys/time.h>
 #endif
 
@@ -137,7 +151,7 @@
 
 /********************************************************/
 
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   extern "C" {
 #endif
 
@@ -164,7 +178,7 @@
 #     if (USE_ETC_FILESYSTEMS)
 #      define DI_MOUNT_FILE     "/etc/filesystems" /* AIX 4.x or /etc/mntent? */
 #     else
-#      define DI_MOUNT_FILE     "/etc/mnttab"       /* SysV.3 default */
+#      define DI_MOUNT_FILE     "/etc/mnttab"      /* SysV.3 default */
 #     endif
 #    endif
 #   endif
@@ -172,24 +186,27 @@
 # endif
 #endif
 
+#if defined (__QNX__)
+static int di_getQNXDiskEntries _((char *ipath, diDiskInfo_t **diskInfo, int *diCount));
+#endif
 extern int debug;
 
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   }
 #endif
 
 /********************************************************/
 
 #if _lib_getmntent && \
-	! _lib_setmntent && \
-	! _lib_mntctl && \
-	! _class_os__Volumes
+    ! _lib_setmntent && \
+    ! _lib_mntctl && \
+    ! _class_os__Volumes
 
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   extern "C" {
 #endif
 static char *checkMountOptions      _((struct mnttab *, char *));
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   }
 #endif
 
@@ -296,6 +313,114 @@ checkMountOptions (mntEntry, str)
 
 #endif
 
+#if _lib_getmntent && \
+    _lib_setmntent && \
+    _lib_endmntent && \
+    ! _lib_getmntinfo && \
+    ! _lib_getfsstat && \
+    ! _lib_getvfsstat && \
+    ! _lib_mntctl && \
+    ! _lib_GetDiskFreeSpace && \
+    ! _lib_GetDiskFreeSpaceEx && \
+    ! _class_os__Volumes
+
+/*
+ * di_getDiskEntries
+ *
+ * SunOS supplies an open and close routine for the mount table.
+ *
+ */
+
+#if ! defined (MNTTYPE_IGNORE)
+# define MNTTYPE_IGNORE "ignore"
+#endif
+
+int
+# if _proto_stdc
+di_getDiskEntries (diDiskInfo_t **diskInfo, int *diCount)
+# else
+di_getDiskEntries (diskInfo, diCount)
+    diDiskInfo_t **diskInfo;
+    int *diCount;
+# endif
+{
+    diDiskInfo_t    *diptr;
+    FILE            *f;
+    int             idx;
+    struct mntent   *mntEntry;
+    char            *devp;   /* local ptr to dev entry */
+
+
+    if (debug > 0) { printf ("# getDiskEntries: set/get/endmntent\n"); }
+/* if both are set not an ansi compiler... */
+#if _setmntent_args == 1
+    if ((f = setmntent (DI_MOUNT_FILE)) == (FILE *) NULL)
+#else
+    if ((f = setmntent (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
+#endif
+    {
+        fprintf (stderr, "Unable to open: %s errno %d\n", DI_MOUNT_FILE, errno);
+        return -1;
+    }
+
+    while ((mntEntry = getmntent (f)) != (struct mntent *) NULL)
+    {
+        idx = *diCount;
+        ++*diCount;
+        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
+                sizeof (diDiskInfo_t) * (Size_t) *diCount);
+        diptr = *diskInfo + idx;
+        di_initDiskInfo (diptr);
+
+        strncpy (diptr->special, mntEntry->mnt_fsname, DI_SPEC_NAME_LEN);
+        strncpy (diptr->name, mntEntry->mnt_dir, DI_NAME_LEN);
+        strncpy (diptr->fsType, mntEntry->mnt_type, DI_TYPE_LEN);
+
+        if (strcmp (mntEntry->mnt_fsname, "none") == 0) {
+          diptr->printFlag = DI_PRNT_IGNORE;
+          if (debug > 2) {
+            printf ("mnt: ignore: special 'none': %s\n", diptr->name);
+          }
+        }
+
+        if (strcmp (mntEntry->mnt_type, MNTTYPE_IGNORE) == 0)
+        {
+            diptr->printFlag = DI_PRNT_IGNORE;
+            if (debug > 2)
+            {
+                printf ("mnt: ignore: mntopt 'ignore': %s\n",
+                        diptr->name);
+            }
+        }
+
+        if ((devp = strstr (mntEntry->mnt_opts, "dev=")) != (char *) NULL)
+        {
+            if (devp != mntEntry->mnt_opts)
+            {
+                --devp;
+            }
+            *devp = 0;   /* point to preceeding comma and cut off */
+        }
+        if (chkMountOptions (mntEntry->mnt_opts, DI_MNTOPT_RO) != (char *) NULL)
+        {
+            diptr->isReadOnly = TRUE;
+        }
+        strncpy (diptr->options, mntEntry->mnt_opts, DI_OPT_LEN);
+
+        if (debug > 1)
+        {
+            printf ("mnt:%s - %s : %s\n", diptr->name,
+                    diptr->special, diptr->fsType);
+        }
+    }
+
+    endmntent (f);
+    return 0;
+}
+
+#endif /* _lib_getmntent && _lib_setmntent && _lib_endmntent */
+
+/* QNX */
 #if ! _lib_getmntent && \
     ! _lib_mntctl && \
     ! _lib_getmntinfo && \
@@ -306,7 +431,157 @@ checkMountOptions (mntEntry, str)
     ! _lib_GetDiskFreeSpaceEx && \
     ! _lib_fs_stat_dev && \
     ! _class_os__Volumes && \
-    ! _lib_sys_dollar_device_scan
+    ! _lib_sys_dollar_device_scan && \
+    defined (__QNX__)
+
+/*
+ * di_getDiskEntries
+ *
+ * QNX
+ *
+ * This is bloody slow.
+ * It would be nice to have a way to short-circuit some of
+ * the directory subtrees.
+ * /proc/mount/dev is not processed...hopefully that won't affect much.
+ *
+ */
+
+int
+# if _proto_stdc
+di_getDiskEntries (diDiskInfo_t **diskInfo, int *diCount)
+# else
+di_getDiskEntries (diskInfo, diCount)
+    diDiskInfo_t **diskInfo;
+    int *diCount;
+# endif
+{
+    if (debug > 0) { printf ("# getDiskEntries: QNX\n"); }
+    return di_getQNXDiskEntries ("/proc/mount", diskInfo, diCount);
+}
+
+static int
+# if _proto_stdc
+di_getQNXDiskEntries (char *ipath, diDiskInfo_t **diskInfo, int *diCount)
+# else
+di_getQNXDiskEntries (ipath, diskInfo, diCount)
+    char *ipath;
+    diDiskInfo_t **diskInfo;
+    int *diCount;
+# endif
+{
+    diDiskInfo_t    *diptr;
+    int             idx;
+    char            path [MAXPATHLEN];
+    int             len;   /* current length of path */
+    DIR             *dirp;
+    struct dirent   *dent;
+    int             ret;
+    int             nodeid;
+    int             pid;
+    int             chid;
+    int             handle;
+    int             ftype;
+    struct stat     statinfo;
+    int             fd;
+    char            tspecial [DI_SPEC_NAME_LEN];
+
+    if (strcmp (ipath, "/proc/mount/dev") == 0) {
+      return 0;
+    }
+
+    strcpy (path, ipath);
+    len = strlen (path);
+
+    if (!(dirp = opendir(path))) {
+      return 0;
+    }
+    while ((dent = readdir(dirp))) {
+      if (strcmp (dent->d_name, ".") == 0 || strcmp (dent->d_name, "..") == 0) {
+        continue;
+      }
+
+      path[len] = '\0';
+      ret = sscanf(dent->d_name, "%d,%d,%d,%d,%d",
+          &nodeid, &pid, &chid, &handle, &ftype);
+
+      if (len + strlen(dent->d_name) + 1 > MAXPATHLEN) {
+        continue;
+      }
+
+      path[len] = '/';
+      strcpy (&path[len+1], dent->d_name);
+      if (debug > 4) { printf ("check: %s\n", path); }
+
+      memset(&statinfo, 0, sizeof(statinfo));
+
+      if (stat(path, &statinfo) == -1) {
+        continue;
+      }
+
+      if (ret != 5) {
+        if (S_ISDIR (statinfo.st_mode)) {
+          if (debug > 4) { printf ("into: %s\n", path); }
+          di_getQNXDiskEntries (path, diskInfo, diCount);
+        }
+        continue;
+      }
+
+      if (ftype != _FTYPE_ANY) {
+        continue;
+      }
+
+      *tspecial = '\0';
+      if (S_ISDIR(statinfo.st_mode) && ftype == _FTYPE_ANY) {
+        if ((fd = open (path, /* O_ACCMODE */ O_RDONLY | O_NOCTTY)) != -1) {
+          devctl (fd, DCMD_FSYS_MOUNTED_ON, tspecial, DI_SPEC_NAME_LEN, 0);
+          close (fd);
+          if (*tspecial == '\0') {
+            /* unfortunately, this cuts out /proc, /dev/sem, etc. */
+            /* but it also removes strange duplicate stuff        */
+            continue;
+          }
+        } else {
+          continue;
+        }
+      } else {
+        continue;
+      }
+
+      idx = *diCount;
+      ++*diCount;
+      *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
+              sizeof (diDiskInfo_t) * (Size_t) *diCount);
+      diptr = *diskInfo + idx;
+      di_initDiskInfo (diptr);
+
+      path[len] = '\0';
+      strncpy (diptr->special, tspecial, DI_SPEC_NAME_LEN);
+      strncpy (diptr->name, path + 11, DI_NAME_LEN);
+      if (*diptr->name == '\0') {
+        strcpy (diptr->name, "/");
+      }
+      if (debug > 4) { printf ("found: %s %s\n", diptr->special, diptr->name); }
+    }
+
+    closedir (dirp);
+    return 0;
+}
+
+#endif /* QNX */
+
+/* if nothing matches, assume a SysV.3 /etc/mnttab or similar */
+#if ! _lib_getmntent && \
+    ! _lib_mntctl && \
+    ! _lib_getmntinfo && \
+    ! _lib_getfsstat && \
+    ! _lib_getvfsstat && \
+    ! _lib_getmnt && \
+    ! _lib_GetDiskFreeSpace && \
+    ! _lib_GetDiskFreeSpaceEx && \
+    ! _lib_fs_stat_dev && \
+    ! _class_os__Volumes && \
+    ! _lib_sys_dollar_device_scan && \
+    ! defined (__QNX__)
 
 /*
  * di_getDiskEntries
@@ -373,120 +648,8 @@ di_getDiskEntries (diskInfo, diCount)
     return 0;
 }
 
-
 #endif /* Sys V.3 */
 
-
-#if _lib_getmntent && \
-    _lib_setmntent && \
-    _lib_endmntent && \
-    ! _lib_getmntinfo && \
-    ! _lib_getfsstat && \
-    ! _lib_getvfsstat && \
-    ! _lib_mntctl && \
-    ! _lib_GetDiskFreeSpace && \
-    ! _lib_GetDiskFreeSpaceEx && \
-    ! _class_os__Volumes
-
-/*
- * di_getDiskEntries
- *
- * SunOS supplies an open and close routine for the mount table.
- *
- */
-
-#if ! defined (MNTTYPE_IGNORE)
-# define MNTTYPE_IGNORE "ignore"
-#endif
-
-int
-# if _proto_stdc
-di_getDiskEntries (diDiskInfo_t **diskInfo, int *diCount)
-# else
-di_getDiskEntries (diskInfo, diCount)
-    diDiskInfo_t **diskInfo;
-    int *diCount;
-# endif
-{
-    diDiskInfo_t     *diptr;
-    FILE            *f;
-    int             idx;
-    struct mntent   *mntEntry;
-    char            *devp;   /* local ptr to dev entry */
-
-
-    if (debug > 0) { printf ("# getDiskEntries: set/get/endmntent\n"); }
-/* if both are set not an ansi compiler... */
-#if _setmntent_args == 1
-    if ((f = setmntent (DI_MOUNT_FILE)) == (FILE *) NULL)
-#else
-    if ((f = setmntent (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
-#endif
-    {
-        fprintf (stderr, "Unable to open: %s errno %d\n", DI_MOUNT_FILE, errno);
-        return -1;
-    }
-
-    while ((mntEntry = getmntent (f)) != (struct mntent *) NULL)
-    {
-        idx = *diCount;
-        ++*diCount;
-        *diskInfo = (diDiskInfo_t *) _realloc ((char *) *diskInfo,
-                sizeof (diDiskInfo_t) * (Size_t) *diCount);
-        diptr = *diskInfo + idx;
-        di_initDiskInfo (diptr);
-
-        strncpy (diptr->special, mntEntry->mnt_fsname, DI_SPEC_NAME_LEN);
-        strncpy (diptr->name, mntEntry->mnt_dir, DI_NAME_LEN);
-        strncpy (diptr->fsType, mntEntry->mnt_type, DI_TYPE_LEN);
-
-        if (strcmp (mntEntry->mnt_fsname, "none") == 0)
-        {
-            diptr->printFlag = DI_PRNT_IGNORE;
-            if (debug > 2)
-            {
-                printf ("mnt: ignore: fstype 'none': %s\n",
-                        diptr->name);
-            }
-        }
-
-        if (strcmp (mntEntry->mnt_type, MNTTYPE_IGNORE) == 0)
-        {
-            diptr->printFlag = DI_PRNT_IGNORE;
-            if (debug > 2)
-            {
-                printf ("mnt: ignore: mntopt 'ignore': %s\n",
-                        diptr->name);
-            }
-        }
-
-        if ((devp = strstr (mntEntry->mnt_opts, "dev=")) != (char *) NULL)
-        {
-            if (devp != mntEntry->mnt_opts)
-            {
-                --devp;
-            }
-            *devp = 0;   /* point to preceeding comma and cut off */
-        }
-        if (chkMountOptions (mntEntry->mnt_opts, DI_MNTOPT_RO) != (char *) NULL)
-        {
-            diptr->isReadOnly = TRUE;
-        }
-        strncpy (diptr->options, mntEntry->mnt_opts, DI_OPT_LEN);
-
-        if (debug > 1)
-        {
-            printf ("mnt:%s - %s : %s\n", diptr->name,
-                    diptr->special, diptr->fsType);
-        }
-    }
-
-    endmntent (f);
-    return 0;
-}
-
-#endif /* _lib_getmntent && _lib_setmntent && _lib_endmntent */
-
 /*
  * All of the following routines also replace di_getDiskInfo()
  */
@@ -509,7 +672,9 @@ di_getDiskEntries (diskInfo, diCount)
 
 # if defined (INITMOUNTNAMES) && ! _dcl_mnt_names
  static char *mnt_names [] = INITMOUNTNAMES;
-#  define MNT_NUMTYPES (MOUNT_MAXTYPE + 1)
+#  if ! defined (MNT_NUMTYPES)
+#   define MNT_NUMTYPES (MOUNT_MAXTYPE + 1)
+#  endif
 # endif
 
 int
@@ -569,10 +734,10 @@ di_getDiskEntries (diskInfo, diCount)
 
         tblocksz = (_fs_size_t) 1024;
 
-# if _mem_f_fsize_statfs /* OSF 1.x */
+# if _mem_struct_statfs_f_fsize /* OSF 1.x */
         tblocksz = (_fs_size_t) mntbufp [idx].f_fsize;
 # endif
-# if _mem_f_bsize_statfs /* OSF 2.x */
+# if _mem_struct_statfs_f_bsize /* OSF 2.x */
         tblocksz = (_fs_size_t) mntbufp [idx].f_bsize;
 # endif
         di_saveBlockSizes (diptr, tblocksz,
@@ -586,7 +751,7 @@ di_getDiskEntries (diskInfo, diCount)
 
         fstype = mntbufp [idx].f_type;
 # if ! _sys_fs_types && ! defined (INITMOUNTNAMES) && \
-    ! _mem_f_fstypename_statfs
+    ! _mem_struct_statfs_f_fstypename
         if ((fstype >= 0) && (fstype <= MOUNT_MAXTYPE))
         {
             switch (fstype)
@@ -720,7 +885,7 @@ di_getDiskEntries (diskInfo, diCount)
             } /* switch on mount type */
         }
 # else
-#  if _mem_f_fstypename_statfs
+#  if _mem_struct_statfs_f_fstypename
         strncpy (diptr->fsType, mntbufp [idx].f_fstypename, DI_TYPE_LEN);
 #  else
             /* could use getvfsbytype here... */
@@ -753,13 +918,13 @@ di_getDiskEntries (diskInfo, diCount)
                     (long) mntbufp [idx].f_blocks,
                     (long) mntbufp [idx].f_bfree,
                     (long) mntbufp [idx].f_bavail);
-# if _mem_f_fsize_statfs
+# if _mem_struct_statfs_f_fsize
             printf ("\tfsize:%ld \n", (long) mntbufp [idx].f_fsize);
 # endif
-# if _mem_f_bsize_statfs
+# if _mem_struct_statfs_f_bsize
             printf ("\tbsize:%ld \n", (long) mntbufp [idx].f_bsize);
 # endif
-# if _mem_f_iosize_statfs
+# if _mem_struct_statfs_f_iosize
             printf ("\tiosize:%ld \n", (long) mntbufp [idx].f_iosize);
 # endif
             printf ("\tinodes: tot:%ld free:%ld\n",
@@ -781,7 +946,7 @@ di_getDiskEntries (diskInfo, diCount)
 /*
  * di_getDiskEntries
  *
- * OSF/1 / Digital Unix / Compaq Tru64 / FreeBSD / NetBSD 2.x
+ * OSF/1 / Digital Unix / Compaq Tru64 / FreeBSD / NetBSD 2.x / OpenBSD
  *
  */
 
@@ -817,7 +982,7 @@ di_getDiskEntries (diskInfo, diCount)
     bufsize = sizeof (struct statfs) * (Size_t) count;
     mntbufp = malloc (bufsize);
     memset ((char *) mntbufp, '\0', sizeof (struct statfs) * (Size_t) count);
-    count = getfsstat (mntbufp, (long) bufsize, MNT_NOWAIT);
+    count = getfsstat (mntbufp, (Size_t) bufsize, MNT_NOWAIT);
 
     *diCount = count;
     *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
@@ -850,7 +1015,7 @@ di_getDiskEntries (diskInfo, diCount)
         }
 # endif
         convertMountOptions ((long) sp->f_flags, diptr);
-# if _mem_f_type_statfs
+# if _mem_struct_statfs_f_type
 #  if defined (MOUNT_NFS3)
         if (sp->f_type == MOUNT_NFS3)
         {
@@ -859,19 +1024,19 @@ di_getDiskEntries (diskInfo, diCount)
         }
 #  endif
 # endif
-# if _mem_mount_info_statfs && \
+# if _mem_struct_statfs_mount_info && \
         defined (MOUNT_NFS) && \
-        (_mem_f_type_statfs || _mem_f_fstypename_statfs)
-#  if _mem_f_type_statfs
+        (_mem_struct_statfs_f_type || _mem_struct_statfs_f_fstypename)
+#  if _mem_struct_statfs_f_type
         if (sp->f_type == MOUNT_NFS
 #  endif
-#  if _mem_f_fstypename_statfs
+#  if _mem_struct_statfs_f_fstypename
         if (strcmp (sp->f_fstypename, MOUNT_NFS) == 0
 #  endif
-#  if _mem_f_fstypename_statfs && defined (MOUNT_NFS3)
+#  if _mem_struct_statfs_f_fstypename && defined (MOUNT_NFS3)
                 || strcmp (sp->f_fstypename, MOUNT_NFS3) == 0
 #  endif
-#  if _mem_f_type_statfs && defined (MOUNT_NFS3)
+#  if _mem_struct_statfs_f_type && defined (MOUNT_NFS3)
                 || sp->f_type == MOUNT_NFS3
 #  endif
            )
@@ -885,10 +1050,10 @@ di_getDiskEntries (diskInfo, diCount)
 
         strncpy (diptr->special, sp->f_mntfromname, DI_SPEC_NAME_LEN);
         strncpy (diptr->name, sp->f_mntonname, DI_NAME_LEN);
-# if _mem_f_fsize_statfs
+# if _mem_struct_statfs_f_fsize
         tblocksz = (_fs_size_t) sp->f_fsize;
 # endif
-# if _mem_f_bsize_statfs && ! _mem_f_fsize_statfs
+# if _mem_struct_statfs_f_bsize && ! _mem_struct_statfs_f_fsize
         tblocksz = (_fs_size_t) sp->f_bsize;
 # endif
         di_saveBlockSizes (diptr, tblocksz,
@@ -899,13 +1064,13 @@ di_getDiskEntries (diskInfo, diCount)
             (_fs_size_t) sp->f_files,
             (_fs_size_t) sp->f_ffree,
             (_fs_size_t) sp->f_ffree);
-# if _mem_f_fstypename_statfs
+# if _mem_struct_statfs_f_fstypename
         strncpy (diptr->fsType, sp->f_fstypename, DI_TYPE_LEN);
 # else
-#  if _lib_sysfs && _mem_f_type_statfs
+#  if _lib_sysfs && _mem_struct_statfs_f_type
         sysfs (GETFSTYP, sp->f_type, diptr->fsType);
 #  else
-#   if _dcl_mnt_names && _mem_f_type_statfs
+#   if _dcl_mnt_names && _mem_struct_statfs_f_type
         fstype = sp->f_type;
         if ((fstype >= 0) && (fstype < MNT_NUMTYPES))
         {
@@ -948,7 +1113,7 @@ di_getDiskEntries (diskInfo, diCount)
     diDiskInfo_t     *diptr;
     int             count;
     int             idx;
-    long            bufsize;
+    Size_t          bufsize;
     struct statvfs  *mntbufp;
     struct statvfs  *sp;
 
@@ -960,9 +1125,9 @@ di_getDiskEntries (diskInfo, diCount)
         return -1;
     }
     bufsize = sizeof (struct statvfs) * count;
-    mntbufp = malloc (bufsize);
+    mntbufp = malloc ((Size_t) bufsize);
     memset ((char *) mntbufp, '\0', sizeof (struct statvfs) * (Size_t) count);
-    count = getvfsstat (mntbufp, bufsize, ST_NOWAIT);
+    count = getvfsstat (mntbufp, (Size_t) bufsize, ST_NOWAIT);
 
     *diCount = count;
     *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
@@ -1224,7 +1389,6 @@ di_getDiskEntries (diskInfo, diCount)
     char            *bufp;      /* pointer into vmbuf                   */
     struct vmount   *vmtp;      /* pointer into vmbuf                   */
     struct vfs_ent  *ve;        /* pointer for file system type entry   */
-    int             len;
 
 
     if (debug > 0) { printf ("# getDiskEntries: mntctl\n"); }
@@ -1236,11 +1400,11 @@ di_getDiskEntries (diskInfo, diCount)
         if ((vmbuf = (char *) malloc (vmbufsz)) == (char *) NULL)
         {
             fprintf (stderr, "malloc (%d) for mntctl() failed errno %d\n",
-                    vmbufsz, errno);
+                    (int) vmbufsz, errno);
             return -1;
         }
 
-        num = mntctl (MCTL_QUERY, vmbufsz, vmbuf);
+        num = mntctl (MCTL_QUERY, (int) vmbufsz, vmbuf);
             /*
              * vmbuf is too small, could happen for
              * following reasons:
@@ -1252,7 +1416,7 @@ di_getDiskEntries (diskInfo, diCount)
             memcpy (&vmbufsz, vmbuf, sizeof (vmbufsz)); /* see mntctl(2) */
             if (debug > 0)
             {
-                printf ("vmbufsz too small, new size: %d\n", vmbufsz);
+                printf ("vmbufsz too small, new size: %d\n", (int) vmbufsz);
             }
             free ((char *) vmbuf); /* free this last, it's still being used! */
             ++i;
@@ -1275,7 +1439,8 @@ di_getDiskEntries (diskInfo, diCount)
 
         /* <num> vmount structs returned in vmbuf */
     *diCount = num;
-    *diskInfo = (diDiskInfo_t *) calloc (sizeof (diDiskInfo_t), *diCount);
+    *diskInfo = (diDiskInfo_t *) calloc (sizeof (diDiskInfo_t),
+        (Size_t) *diCount);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. %s errno %d\n",
@@ -1344,6 +1509,7 @@ di_getDiskEntries (diskInfo, diCount)
             printf ("\t%s\n", (char *) vmt2dataptr (vmtp, VMT_ARGS));
         }
     }
+    return 0;
 }
 
 #endif  /* _lib_mntctl */
@@ -1374,7 +1540,7 @@ di_getDiskEntries (diskInfo, diCount)
 {
     diDiskInfo_t     *diptr;
     int             i;
-    int             diskflag;
+    char            diskflag;
     int             rc;
     char            *p;
     char            buff [MSDOS_BUFFER_SIZE];
@@ -1388,10 +1554,11 @@ di_getDiskEntries (diskInfo, diCount)
 #  endif
 # endif
     diskflag = DI_PRNT_IGNORE;
-    rc = GetLogicalDriveStrings (MSDOS_BUFFER_SIZE, buff);
+    rc = (int) GetLogicalDriveStrings (MSDOS_BUFFER_SIZE, buff);
     *diCount = rc / BYTES_PER_LOGICAL_DRIVE;
 
-    *diskInfo = (diDiskInfo_t *) calloc (sizeof (diDiskInfo_t), *diCount);
+    *diskInfo = (diDiskInfo_t *)
+        calloc (sizeof (diDiskInfo_t), (Size_t) *diCount);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
@@ -1403,7 +1570,7 @@ di_getDiskEntries (diskInfo, diCount)
         diptr = *diskInfo + i;
         p = buff + (BYTES_PER_LOGICAL_DRIVE * i);
         strncpy (diptr->name, p, DI_NAME_LEN);
-        rc = GetDriveType (p);
+        rc = (int) GetDriveType (p);
         diptr->printFlag = DI_PRNT_OK;
 
         if (rc == DRIVE_NO_ROOT_DIR)
