@@ -1,5 +1,5 @@
 /*
- * $Id: digetinfo.c,v 1.12 2010-03-14 16:41:48-07 bll Exp $
+ * $Id: digetinfo.c,v 1.17 2010-07-14 01:54:42-07 bll Exp $
  * $Source: /home/bll/DI/di/RCS/digetinfo.c,v $
  * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
  */
@@ -22,7 +22,9 @@
 #include "di.h"
 #include "dimntopt.h"
 
-#include <stdio.h>
+#if _hdr_stdio
+# include <stdio.h>
+#endif
 #if _hdr_stdlib
 # include <stdlib.h>
 #endif
@@ -65,7 +67,7 @@
 
 /********************************************************/
 
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   extern "C" {
 #endif
 
@@ -85,7 +87,7 @@
 
 extern int debug;
 
-#if defined(__cplusplus)
+#if defined (__cplusplus) || defined (c_plusplus)
   }
 #endif
 
@@ -157,6 +159,11 @@ di_getDiskInfo (diskInfo, diCount)
                     (_fs_size_t) statBuf.f_files,
                     (_fs_size_t) statBuf.f_ffree,
                     (_fs_size_t) statBuf.f_favail);
+# if _mem_struct_statvfs_f_basetype
+                if (! *diptr->fsType) {
+                  strncpy (diptr->fsType, statBuf.f_basetype, DI_TYPE_LEN);
+                }
+# endif
 
                 if (debug > 1)
                 {
@@ -243,7 +250,7 @@ di_getDiskInfo (diskInfo, diCount)
 
             if (statfs (diptr->name, &statBuf, sizeof (statBuf), 0) == 0)
             {
-# if _mem_f_frsize_statfs
+# if _mem_struct_statfs_f_frsize
                 if (statBuf.f_frsize == 0 && statBuf.f_bsize != 0)
                 {
                     tblocksz = (_fs_size_t) statBuf.f_bsize;
@@ -263,14 +270,14 @@ di_getDiskInfo (diskInfo, diCount)
                     (_fs_size_t) statBuf.f_files,
                     (_fs_size_t) statBuf.f_ffree,
                     (_fs_size_t) statBuf.f_ffree);
-# if _lib_sysfs && _mem_f_fstyp_statfs
+# if _lib_sysfs && _mem_struct_statfs_f_fstyp
                 sysfs (GETFSTYP, statBuf.f_fstyp, diptr->fsType);
 # endif
 
                 if (debug > 1)
                 {
                     printf ("%s: %s\n", diptr->name, diptr->fsType);
-# if _mem_f_frsize_statfs
+# if _mem_struct_statfs_f_frsize
                     printf ("\tbsize:%ld\n", statBuf.f_bsize);
                     printf ("\tfrsize:%ld\n", statBuf.f_frsize);
 # else
@@ -342,7 +349,7 @@ di_getDiskInfo (diskInfo, diCount)
                     (_fs_size_t) statBuf.f_ffree,
                     (_fs_size_t) statBuf.f_ffree);
 
-# if _lib_sysfs && _mem_f_fstyp_statfs
+# if _lib_sysfs && _mem_struct_statfs_f_fstyp
                 sysfs (GETFSTYP, statBuf.f_fstyp, diptr->fsType);
 # endif
 
