@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: mkconfig.pl,v 1.62 2010-03-12 00:21:52-08 bll Exp $
+# $Id: mkconfig.pl,v 1.83 2010-07-19 07:25:53-07 bll Exp $
 #
 # Copyright 2006-2010 Brad Lanam Walnut Creek, CA USA
 #
@@ -13,27 +13,34 @@ require 5.005;
 my $CONFH;
 my $LOG = "mkconfig.log";
 my $TMP = "_tmp_mkconfig";
+my $OPTIONFILE = "options.dat";
 my $CACHEFILE = "mkconfig.cache";
 my $VARSFILE = "mkconfig_c.vars";
-my $REQLIB = "reqlibs.txt";
+my $REQLIB = "mkconfig.reqlibs";
 
 my $precc = <<'_HERE_';
 #if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
-# define _ARG_(x) x
-# define _VOID_ void
+# define _(x) x
 #else
-# define _ARG_(x) ()
-# define _VOID_ char
+# define _(x) ()
+# define void char
 #endif
-#if defined(__cplusplus)
-# define _BEGIN_EXTERNS_ extern "C" {
-# define _END_EXTERNS_ }
+#if defined(__cplusplus) || defined (c_plusplus)
+# define CPP_EXTERNS_BEG extern "C" {
+# define CPP_EXTERNS_END }
+CPP_EXTERNS_BEG
+extern int printf (const char *, ...);
+CPP_EXTERNS_END
 #else
-# define _BEGIN_EXTERNS_
-# define _END_EXTERNS_
+# define CPP_EXTERNS_BEG
+# define CPP_EXTERNS_END
 #endif
 _HERE_
 
+my $optionsloaded = 0;
+my %optionshash;
+my $iflevels = '';
+
 sub
 exitmkconfig
 {
@@ -45,25 +52,29 @@ sub
 printlabel
 {
     my ($name, $label) = @_;
-    print LOGFH "## [$name] $label ... \n";
-    print STDOUT "$label ... ";
+    print LOGFH "## ${iflevels}[$name] $label ... \n";
+    print STDOUT "${iflevels}$label ... ";
+}
+
+sub
+printyesno_actual
+{
+    my ($name, $val, $tag) = @_;
+
+    print LOGFH "## [$name] $val $tag\n";
+    print STDOUT "$val $tag\n";
 }
 
 sub
 printyesno_val
 {
-    my ($name, $val, $tag) = @_;
+  my ($name, $val, $tag) = @_;
 
-    if ($val ne "0")
-    {
-        print LOGFH "## [$name] $val $tag\n";
-        print STDOUT "$val $tag\n";
-    }
-    else
-    {
-        print LOGFH "## [$name] no $tag\n";
-        print STDOUT "no $tag\n";
-    }
+  if ($val ne '0') {
+    printyesno_actual ($name, $val, $tag);
+  } else {
+    printyesno_actual ($name, 'no', $tag);
+  }
 }
 
 sub
@@ -87,8 +98,14 @@ savecache
     open (MKCV, ">$VARSFILE");
     foreach my $val (@{$r_clist->{'list'}})
     {
-      print MKCC "di_c_${val}='" . $r_config->{$val} . "'\n";
-      print MKCV $val, "\n";
+      if ($val =~ /^lib__lib_/o) {
+        $tval = $val;
+        $tval =~ s/^lib_//o;
+        print MKCC "di_c_lib_${tval}='" . $r_config->{$val} . "'\n";
+      } else {
+        print MKCC "di_c_${val}='" . $r_config->{$val} . "'\n";
+        print MKCV $val, "\n";
+      }
     }
     close (MKCC);
     close (MKCV);
@@ -122,6 +139,24 @@ checkcache
   return $rc;
 }
 
+sub
+loadoptions
+{
+  if ($optionsloaded == 0 && open (OPTS, "<$OPTIONFILE")) {
+    while (my $o = <OPTS>) {
+      chomp $o;
+      if ($o =~ /^$/o || $o =~ /^#/o) {
+        next;
+      }
+      my ($onm, $val) = split (/=/, $o);
+      printf LOGFH "## load: $onm = $val\n";
+      $optionshash{$onm} = $val;
+    }
+    $optionsloaded = 1;
+    close (OPTS);
+  }
+}
+
 sub
 setlist
 {
@@ -172,8 +207,14 @@ print_headers
                 next;
             }
             if ($val eq '_sys_time' &&
-                $r_config->{'_hdr_time'} ne '0' &&
-                $r_config->{'_include_time'} eq '0')
+                $r_config->{'_sys_time'} ne '0' &&
+                $r_config->{'_inc_conflict__hdr_time__sys_time'} eq '0')
+            {
+                next;
+            }
+            if ($val eq '_hdr_linux_quota' &&
+                $r_config->{'_hdr_linux_quota'} ne '0' &&
+                $r_config->{'_inc_conflict__sys_quota__hdr_linux_quota'} eq '0')
             {
                 next;
             }
@@ -189,11 +230,11 @@ print_headers
 }
 
 sub
-check_run
+_chk_run
 {
     my ($name, $code, $r_val, $r_clist, $r_config, $r_a) = @_;
 
-    my $rc = check_link ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', %$r_a, });
     print LOGFH "##  run test: link: $rc\n";
     $$r_val = 0;
@@ -201,6 +242,7 @@ check_run
     {
         $rc = system ("./$name.exe > $name.out");
         if ($rc & 127) { exitmkconfig ($rc); }
+        $rc >>= 8;
         print LOGFH "##  run test: run: $rc\n";
         if ($rc == 0)
         {
@@ -214,7 +256,7 @@ check_run
 }
 
 sub
-check_link
+_chk_link
 {
     my ($name, $code, $r_clist, $r_config, $r_a) = @_;
 
@@ -236,17 +278,17 @@ check_link
     if ($rc & 127) { exitmkconfig ($rc); }
 
     my $dlibs = '';
-    $rc = _check_link ($name, {} );
+    $rc = _chk_link_libs ($name, {} );
     if ($rc != 0)
     {
       if ($otherlibs ne '')
       {
-        my @olibs = split (/\s+/, $otherlibs);
+        my @olibs = split (/,/, $otherlibs);
         my $oliblist = '';
         foreach my $olib (@olibs)
         {
           $oliblist = $oliblist . ' ' . $olib;
-          $rc = _check_link ($name, { 'otherlibs' => $oliblist, } );
+          $rc = _chk_link_libs ($name, { 'otherlibs' => $oliblist, } );
           if ($rc == 0)
           {
 
@@ -278,11 +320,11 @@ check_link
 }
 
 sub
-_check_link
+_chk_link_libs
 {
     my ($name, $r_a) = @_;
 
-    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} ";
+    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} ";
     if (defined ($r_a->{'cflags'}))
     {
         $cmd .= ' ' . $r_a->{'cflags'} . ' ';
@@ -308,7 +350,36 @@ _check_link
 }
 
 sub
-check_compile
+_check_cpp
+{
+    my ($name, $code, $r_a, $r_clist, $r_config) = @_;
+
+    open (CCFH, ">$name.c");
+    print CCFH $precc;
+    print CCFH $code;
+    close CCFH;
+
+    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} ";
+    if (defined ($r_a->{'cflags'}))
+    {
+        $cmd .= ' ' . $r_a->{'cflags'} . ' ';
+    }
+    $cmd .= "-E $name.c > $name.out ";
+    my $rc = system ("cat $name.c >> $LOG");
+    if ($rc & 127) { exitmkconfig ($rc); }
+    $rc = system ("$cmd 2>>$LOG");
+    print LOGFH "##  cpp test: $cmd\n";
+    if ($rc & 127) { exitmkconfig ($rc); }
+    print LOGFH "##      cpp test: $rc\n";
+    if ($rc == 0 && ! -f "$name.out") {
+      print LOGFH "##      can't locate $name.out\n";
+      $rc = -2;
+    }
+    return $rc;
+}
+
+sub
+_chk_compile
 {
     my ($name, $code, $r_clist, $r_config, $r_a) = @_;
 
@@ -321,7 +392,7 @@ check_compile
     print CCFH $code;
     close CCFH;
 
-    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} -c $name.c";
+    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} -c $name.c";
     print LOGFH "##  compile test: $cmd\n";
     my $rc = system ("cat $name.c >> $LOG");
     if ($rc & 127) { exitmkconfig ($rc); }
@@ -332,11 +403,11 @@ check_compile
 }
 
 sub
-do_check_compile
+do_chk_compile
 {
     my ($name, $code, $inc, $r_clist, $r_config) = @_;
 
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => $inc, });
     my $trc = 0;
     if ($rc == 0)
@@ -369,10 +440,10 @@ _HERE_
     }
     $code .= <<"_HERE_";
 #include <${file}>
-main () { exit (0); }
+main () { return (0); }
 _HERE_
     my $rc = 1;
-    $rc = check_compile ($name, $code, $r_clist, $r_config,
+    $rc = _chk_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'std', });
     my $val = 0;
     if ($rc == 0)
@@ -404,9 +475,9 @@ check_constant
 _HERE_
     }
     $code .= <<"_HERE_";
-main () { if (${constant} == 0) { 1; } exit (0); }
+main () { if (${constant} == 0) { 1; } return (0); }
 _HERE_
-    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
+    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 # if the keyword is reserved, the compile will fail.
@@ -423,9 +494,9 @@ check_keyword
 
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
-main () { int ${keyword}; ${keyword} = 1; exit (0); }
+main () { int ${keyword}; ${keyword} = 1; return (0); }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'std', });
     setlist $r_clist, $name;
     if ($rc != 0)  # failure means it is reserved...
@@ -447,12 +518,12 @@ check_proto
     }
 
     my $code = <<"_HERE_";
-_BEGIN_EXTERNS_
+CPP_EXTERNS_BEG
 extern int foo (int, int);
-_END_EXTERNS_
+CPP_EXTERNS_END
 int bar () { int rc; rc = foo (1,1); return 0; }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', });
     setlist $r_clist, $name;
     $r_config->{$name} = 0;
@@ -488,30 +559,243 @@ check_command
 }
 
 sub
-check_include_time
+check_ifoption
 {
-    my ($name, $r_clist, $r_config) = @_;
+    my ($ifcount, $type, $name, $opt, $r_clist, $r_config) = @_;
 
-    if (defined ($r_config->{'_hdr_time'}) &&
-        $r_config->{'_hdr_time'} ne '0' &&
-        defined ($r_config->{'_sys_time'}) &&
-        $r_config->{'_sys_time'} ne '0')
+    printlabel $name, "$type ($ifcount): $opt";
+
+    loadoptions ();
+
+    my $trc = 'F';
+
+    my $found = 'F';
+    if ($optionsloaded && defined ($optionshash{$opt})) {
+      $found = 'T';
+      $trc = lc $optionshash{$opt};
+      print LOGFH "##  found: $opt => $trc\n";
+      if ($trc eq 't') { $trc = 'T'; }
+      if ($trc eq 'enable') { $trc = 'T'; }
+      if ($trc eq 'f') { $trc = 'F'; }
+      if ($trc eq 'disable') { $trc = 'F'; }
+      if ($trc eq 'true') { $trc = 'T'; }
+      if ($trc eq 'false') { $trc = 'F'; }
+    }
+
+    if ($trc eq 'T') { $trc = 1; }
+    if ($trc eq 'F') { $trc = 0; }
+
+    if ($type eq 'ifnotoption') {
+      $trc = $trc == 0 ? 1 : 0;
+    }
+
+    if (! $optionsloaded) {
+      $trc = 0;
+      printyesno_actual $name, "no options file";
+    } elsif ($found eq 'F') {
+      $trc = 0;
+      printyesno_actual $name, "option not found";
+    } else {
+      printyesno $name, $trc;
+    }
+    return $trc;
+}
+
+sub
+check_if
+{
+    my ($iflabel, $ifcount, $ifline, $r_clist, $r_config) = @_;
+
+    my $name = "_if_$iflabel";
+    my $trc = 0;
+    # make parseable
+    $ifline =~ s/!/ ! /og;
+    $ifline =~ s/\(/ ( /og;
+    $ifline =~ s/\)/ ) /og;
+    $ifline =~ s/&&/ \&\& /og;
+    $ifline =~ s/\|\|/ || /og;
+    $ifline =~ s/ and / && /og;
+    $ifline =~ s/ or / || /og;
+    $ifline =~ s/ not / ! /og;
+    $ifline =~ s/ +/ /og;
+    $ifline =~ s/^ *//og;
+    $ifline =~ s/ *$//og;
+
+    printlabel $name, "if ($ifcount): $iflabel";
+    my $nline = '';
+    print LOGFH "## ifline: $ifline\n";
+    foreach my $token (split (/\s/, $ifline)) {
+      if ($token eq '(' || $token eq ')' || $token eq '&&' ||
+          $token eq '||' || $token eq '!') {
+        $nline .= $token . ' ';
+      } else {
+        print LOGFH "## value of $token: " . $r_config->{$token} . "\n";
+        $nline .= $r_config->{$token} ne '0' ? 1 : 0;
+        $nline .= ' ';
+      }
+    }
+    print LOGFH "## nline: $nline\n";
+    $trc = eval $nline;
+    printyesno $name, $trc;
+    return $trc;
+}
+
+sub
+check_set
+{
+    my ($name, $type, $val, $r_clist, $r_config) = @_;
+
+    my $tnm = $name;
+    $tnm =~ s/^_setint_//;
+    $tnm =~ s/^_setstr_//;
+    printlabel $name, "${type}: $tnm";
+
+    if ($type eq 'set') {
+      if (defined ($r_config->{$name})) {
+        setlist $r_clist, $name;
+        $r_config->{$name} = $val;
+        printyesno $name, $r_config->{$name};
+      } else {
+        printyesno_actual $name, 'no such variable';
+      }
+    } elsif ($type eq 'setint') {
+      setlist $r_clist, $name;
+      $r_config->{$name} = $val;
+      printyesno_actual $name, $r_config->{$name};
+    } else {
+      setlist $r_clist, $name;
+      $r_config->{$name} = $val;
+      printyesno_actual $name, $r_config->{$name};
+    }
+}
+
+sub
+check_option
+{
+    my ($name, $onm, $def, $r_clist, $r_config) = @_;
+
+    printlabel $name, "option: $onm";
+
+    loadoptions ();
+    my $oval = $def;
+
+    if ($optionsloaded && defined ($optionshash{$onm})) {
+      $oval = $optionshash{$onm};
+      print LOGFH "##  found: $onm => $oval\n";
+    }
+
+    setlist $r_clist, $name;
+    $r_config->{$name} = $oval;
+    printyesno_actual $name, $r_config->{$name};
+}
+
+sub
+check_quotactl_pos
+{
+  my ($name, $r_clist, $r_config) = @_;
+
+  printlabel $name, "quotactl position";
+  if (checkcache ($name, $r_config) == 0) {
+    return;
+  }
+
+  setlist $r_clist, $name;
+
+  if ($r_config->{'_lib_quotactl'} eq '0') {
+    $r_config->{$name} = '0';
+    printyesno_val $name, $r_config->{$name};
+    return;
+  }
+
+  my $hdr = '';
+  foreach my $hnm ('_sys_quota', '_hdr_ufs_ufs_quota', '_hdr_ufs_quota',
+      '_hdr_linux_quota') {
+    if ($r_config->{$hnm} ne '0') {
+      $hdr = $r_config->{$hnm};
+      last;
+    }
+  }
+  my $uhdr = $r_config->{'_hdr_unistd'};
+
+  my $code = '';
+  if ($hdr ne '') {
+    $code .= "#include <${hdr}>\n";
+    if ($uhdr ne '0') {
+      $code .= "#include <${uhdr}>\n";
+    }
+    my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
+    if ($rc == 0) {
+       $cmd = "egrep -l 'quotactl *\\( *int[^,]*, *(const *)?char' $name.out >>$LOG 2>&1";
+       print LOGFH "##  quotactl_pos: $cmd\n";
+       $rc = system ($cmd);
+       if ($rc == 0) {
+         $r_config->{$name} = 2;
+         printyesno_val $name, $r_config->{$name};
+         return;
+       }
+
+       $cmd = "egrep -l 'quotactl *\\( *(const *)?char[^,]*, *int' $name.out >>$LOG 2>&1";
+       print LOGFH "##  quotactl_pos: $cmd\n";
+       $rc = system ($cmd);
+       if ($rc == 0) {
+         $r_config->{$name} = 1;
+         printyesno_val $name, $r_config->{$name};
+         return;
+       }
+    }
+  }
+
+  $r_config->{$name} = 0;
+  printyesno_val $name, $r_config->{$name};
+}
+
+sub
+check_include_conflict
+{
+    my ($name, $h1, $h2, $r_clist, $r_config) = @_;
+
+    my $i1 = $h1;
+    $i1 =~ s,/,_,go;
+    $i1 =~ s,\.h$,,o;
+    $i1 =~ s,:,,go;
+    if ($i1 =~ m#^sys#o) {
+      $i1 = "_${i1}";
+    } else {
+      $i1 = "_hdr_${i1}";
+    }
+    my $i2 = $h2;
+    $i2 =~ s,/,_,go;
+    $i2 =~ s,\.h$,,o;
+    $i2 =~ s,:,,go;
+    if ($i2 =~ m#^sys#o) {
+      $i2 = "_${i2}";
+    } else {
+      $i2 = "_hdr_${i2}";
+    }
+
+    $name = "${name}_${i1}_${i2}";
+    printlabel $name, "header: include both $h1 & $h2";
+
+    if (defined ($r_config->{${i1}}) &&
+        $r_config->{${i1}} ne '0' &&
+        defined ($r_config->{${i2}}) &&
+        $r_config->{${i2}} ne '0')
     {
-        printlabel $name, "header: include both time.h & sys/time.h";
         if (checkcache ($name, $r_config) == 0)
         {
             return;
         }
 
         my $code = <<"_HERE_";
-#include <time.h>
-#include <sys/time.h>
-main () { struct tm x; }
+#include <$h1>
+#include <$h2>
+main () { return 0; }
 _HERE_
-        do_check_compile ($name, $code, 'std', $r_clist, $r_config);
+        do_chk_compile ($name, $code, 'std', $r_clist, $r_config);
     } else {
         setlist $r_clist, $name;
-        $r_config->{$name} = 0;
+        $r_config->{$name} = 1;
+        printyesno $name, $r_config->{$name};
     }
 }
 
@@ -535,12 +819,12 @@ check_npt
     }
 
     my $code = <<"_HERE_";
-_BEGIN_EXTERNS_
+CPP_EXTERNS_BEG
 struct _TEST_struct { int _TEST_member; };
-extern struct _TEST_struct* $proto _ARG_((struct _TEST_struct*));
-_END_EXTERNS_
+extern struct _TEST_struct* $proto _((struct _TEST_struct*));
+CPP_EXTERNS_END
 _HERE_
-    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
+    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 sub
@@ -548,6 +832,7 @@ check_type
 {
     my ($name, $type, $r_clist, $r_config) = @_;
 
+    $type =~ s/star/*/og;
     printlabel $name, "type: $type";
     if (checkcache ($name, $r_config) == 0)
     {
@@ -558,9 +843,59 @@ check_type
 struct xxx { $type mem; };
 static struct xxx v;
 struct xxx* f() { return &v; }
-main () { struct xxx *tmp; tmp = f(); exit (0); }
+main () { struct xxx *tmp; tmp = f(); return (0); }
 _HERE_
-    do_check_compile ($name, $code, 'all', $r_clist, $r_config);
+    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
+}
+
+sub
+check_defined
+{
+    my ($name, $def, $r_clist, $r_config) = @_;
+
+    printlabel $name, "defined: $def";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    setlist $r_clist, $name;
+    my $code = <<"_HERE_";
+main () {
+#ifdef ${def}
+return (0);
+#else
+return (1);
+#endif
+}
+_HERE_
+    my $val = 0;
+    my $rc = _chk_run ($name, $code, \$val, $r_clist, $r_config, {});
+    $r_config->{$name} = $rc == 0 ? 1 : 0;
+    printyesno $name, $r_config->{$name};
+}
+
+sub
+check_param_void_star
+{
+    my ($name, $r_clist, $r_config) = @_;
+
+    printlabel $name, "parameter: void *";
+    if (checkcache ($name, $r_config) == 0)
+    {
+        return;
+    }
+
+    my $code = <<"_HERE_";
+char *
+tparamvs (ptr)
+  void *ptr;
+{
+  ptr = (void *) NULL;
+  return (char *) ptr;
+}
+_HERE_
+    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
 }
 
 sub
@@ -599,7 +934,7 @@ _HERE_
          'incheaders' => 'all',
          'otherlibs' => $val,
          );
-    my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
+    my $rc = _chk_link ($name, $code, $r_clist, $r_config, \%a);
     my $tag = '';
     if ($rc == 0)
     {
@@ -607,6 +942,8 @@ _HERE_
       if ($a{'dlibs'} ne '')
       {
           $tag = " with $a{'dlibs'}";
+          $r_config->{"lib_$name"} = $a{'dlibs'};
+          setlist $r_clist, "lib_$name";
       }
     }
     printyesno $name, $r_config->{$name}, $tag;
@@ -642,7 +979,7 @@ _HERE_
          'incheaders' => 'all',
          'otherlibs' => $val,
          );
-    my $rc = check_link ($name, $code, $r_clist, $r_config, \%a);
+    my $rc = _chk_link ($name, $code, $r_clist, $r_config, \%a);
     my $tag = '';
     if ($rc == 0)
     {
@@ -650,6 +987,8 @@ _HERE_
       if ($a{'dlibs'} ne '')
       {
           $tag = " with $a{'dlibs'}";
+          $r_config->{"lib_$name"} = $a{'dlibs'};
+          setlist $r_clist, "lib_$name";
       }
     }
     printyesno $name, $r_config->{$name}, $tag;
@@ -677,7 +1016,7 @@ check_setmntent_args
     my $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab"); }
 _HERE_
-    my $rc = check_link ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', 'otherlibs' => undef, });
     if ($rc == 0)
     {
@@ -689,7 +1028,7 @@ _HERE_
     $code = <<"_HERE_";
 main () { setmntent ("/etc/mnttab", "r"); }
 _HERE_
-    $rc = check_link ($name, $code, $r_clist, $r_config,
+    $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', 'otherlibs' => undef, });
     if ($rc == 0)
     {
@@ -724,7 +1063,7 @@ main () {
     statfs (name, &statBuf);
 }
 _HERE_
-    my $rc = check_link ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', 'otherlibs' => undef, });
     if ($rc == 0)
     {
@@ -739,7 +1078,7 @@ main () {
     statfs (name, &statBuf, sizeof (statBuf));
 }
 _HERE_
-    $rc = check_link ($name, $code, $r_clist, $r_config,
+    $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', 'otherlibs' => undef, });
     if ($rc == 0)
     {
@@ -754,7 +1093,7 @@ main () {
     statfs (name, &statBuf, sizeof (statBuf), 0);
 }
 _HERE_
-    $rc = check_link ($name, $code, $r_clist, $r_config,
+    $rc = _chk_link ($name, $code, $r_clist, $r_config,
         { 'incheaders' => 'all', 'otherlibs' => undef, });
     if ($rc == 0)
     {
@@ -779,11 +1118,11 @@ check_size
     my $code = <<"_HERE_";
 main () {
 	printf("%u\\n", sizeof($type));
-    exit (0);
+    return (0);
     }
 _HERE_
     my $val = 0;
-    my $rc = check_run ($name, $code, \$val, $r_clist, $r_config, {});
+    my $rc = _chk_run ($name, $code, \$val, $r_clist, $r_config, {});
     if ($rc == 0)
     {
         $r_config->{$name} = $val;
@@ -805,9 +1144,9 @@ check_member
     setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
-main () { struct $struct s; int i; i = sizeof (s.$member); }
+main () { $struct s; int i; i = sizeof (s.$member); }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
@@ -832,7 +1171,7 @@ check_int_declare
     my $code = <<"_HERE_";
     main () { int x; x = $function; }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
@@ -855,9 +1194,9 @@ check_ptr_declare
     setlist $r_clist, $name;
     $r_config->{$name} = 0;
     my $code = <<"_HERE_";
-main () { _VOID_ *x; x = $function; }
+main () { void *x; x = $function; }
 _HERE_
-    my $rc = check_compile ($name, $code, $r_clist, $r_config,
+    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
             { 'incheaders' => 'all', });
     if ($rc == 0)
     {
@@ -866,15 +1205,112 @@ _HERE_
     printyesno $name, $r_config->{$name};
 }
 
+sub
+check_standard
+{
+  my ($r_clist, $r_config) = @_;
+  # FreeBSD has buggy headers, requires sys/param.h as a required include.
+  # always check for these headers.
+  my @headlist1 = (
+      [ "_hdr_stdio", "stdio.h", ],
+      [ "_hdr_stdlib", "stdlib.h", ],
+      [ "_sys_types", "sys/types.h", ],
+      [ "_sys_param", "sys/param.h", ],
+      );
+
+  foreach my $r_arr (@headlist1)
+  {
+      check_header ($$r_arr[0], $$r_arr[1], $r_clist, $r_config,
+              { 'reqhdr' => [], });
+  }
+  check_keyword ('_key_void', 'void', $r_clist, $r_config);
+  check_keyword ('_key_const', 'const', $r_clist, $r_config);
+  check_param_void_star ('_param_void_star', $r_clist, $r_config);
+  check_proto ('_proto_stdc', $r_clist, $r_config);
+}
+
 
 sub
-create_config
+create_output
+{
+  my ($r_clist, $r_config, $include) = @_;
+
+  if ($CONFH eq 'none') { return; }
+
+  open (CCOFH, ">$CONFH");
+  print CCOFH <<'_HERE_';
+#ifndef __INC_CONFIG_H
+#define __INC_CONFIG_H 1
+
+_HERE_
+
+  foreach my $val (@{$r_clist->{'list'}})
+  {
+    if ($val =~ m#^lib__lib_#o) {
+      next;
+    }
+    my $tval = 0;
+    if ($r_config->{$val} ne "0") {
+        $tval = 1;
+    }
+    if ($val =~ m#^_setint_#o) {
+      $tnm = $val;
+      $tnm =~ s/^_setint_//;
+      print CCOFH "#define $tnm " . $r_config->{$val} . "\n";
+    } elsif ($val =~ m#^_setstr_#o || $val =~ m#^_opt_#o) {
+      $tnm = $val;
+      $tnm =~ s/^_setstr_//;
+      $tnm =~ s/^_opt_//;
+      print CCOFH "#define $tnm \"" . $r_config->{$val} . "\"\n";
+    } elsif ($val =~ m#^(_hdr|_sys|_command)#o) {
+      print CCOFH "#define $val $tval\n";
+    } else {
+      print CCOFH "#define $val " . $r_config->{$val} . "\n";
+    }
+  }
+
+  # standard tail -- always needed; non specific
+  print CCOFH <<'_HERE_';
+
+#if ! _key_void
+# define void int
+#endif
+#if ! _key_void || ! _param_void_star
+  typedef char *_pvoid;
+#else
+  typedef void *_pvoid;
+#endif
+#if ! _key_const
+# define const
+#endif
+
+#ifndef _
+# if _proto_stdc
+#  define _(args) args
+# else
+#  define _(args) ()
+# endif
+#endif
+
+_HERE_
+
+  print CCOFH $include;
+
+  print CCOFH <<'_HERE_';
+
+#endif /* __INC_CONFIG_H */
+_HERE_
+  close CCOFH;
+}
+
+sub
+main_process
 {
     my ($configfile) = @_;
     my (%clist, %config);
 
     $clist{'list'} = ();
-    $clist{'hash'} = ();
+    $clist{'hash'} = {};
     $config{'reqlibs'} = {};
     $config{'reqlibs_list'} = ();
 
@@ -903,25 +1339,11 @@ create_config
     }
 
 
-    # FreeBSD has buggy headers, requires sys/param.h as a required include.
-    # always check for these headers.
-    my @headlist1 = (
-        [ "_hdr_stdio", "stdio.h", ],
-        [ "_hdr_stdlib", "stdlib.h", ],
-        [ "_sys_types", "sys/types.h", ],
-        [ "_sys_param", "sys/param.h", ],
-        );
-
-    foreach my $r_arr (@headlist1)
-    {
-        check_header ($$r_arr[0], $$r_arr[1], \%clist, \%config,
-                { 'reqhdr' => [], });
+    my $tconfigfile = $configfile;
+    if ($configfile !~ m#^/#) {
+      $tconfigfile = "../$configfile";
     }
-    check_keyword ('_key_void', 'void', \%clist, \%config);
-    check_keyword ('_key_const', 'const', \%clist, \%config);
-    check_proto ('_proto_stdc', \%clist, \%config);
-
-    if (! open (DATAIN, "<../$configfile"))
+    if (! open (DATAIN, "<$tconfigfile"))
     {
         print STDOUT "$configfile: $!\n";
         exit 1;
@@ -930,85 +1352,153 @@ create_config
     my $linenumber = 0;
     my $inheaders = 1;
     my $ininclude = 0;
+    my @doproclist;
+    my $inproc = 0;
+    my $doproc = 1;
     my $include = '';
+    my $tline = '';
+    my $ifstmtcount = 0;
     while (my $line = <DATAIN>)
     {
-        chomp $line;
-        ++$linenumber;
+      chomp $line;
+      ++$linenumber;
 
-        if ($ininclude == 0 && ($line =~ /^#/o || $line eq ''))
-        {
-            next;
+      if ($ininclude == 0 && ($line =~ /^#/o || $line eq ''))
+      {
+          next;
+      }
+
+      if ($tline ne '') {
+        $line = $tline . ' ' . $line;
+        $tline = '';
+      }
+      if ($line =~ /[^\\]\\$/o) {
+        $tline = $line;
+        $tline =~ s/\\$//o;
+        next;
+      }
+
+      if ($ininclude == 1 && $line =~ m#^\s*endinclude$#o)
+      {
+          print LOGFH "end include\n";
+          $ininclude = 0;
+          next;
+      }
+      elsif ($ininclude == 1)
+      {
+          $line =~ s,\\(.),$1,g;
+          $include .= $line . "\n";
+          next;
+      }
+
+      if ($inheaders && $line !~ m#^\s*(hdr|sys)#o)
+      {
+          $inheaders = 0;
+      }
+
+      print LOGFH "#### ${linenumber}: ${line}\n";
+
+      if ($line =~ m#^\s*else#o)
+      {
+          $doproc = $doproc == 0 ? 1 : 0;
+          $iflevels =~ s/.\d+\s$//;
+          $iflevels .= "-$ifstmtcount ";
+      }
+      elsif ($line =~ m#^\s*endif#o)
+      {
+        if ($#doproclist >= 0) {
+          print LOGFH "## endif: doproclist: " . join (' ', @doproclist) . "\n";
+          $doproc = shift @doproclist;
+          print LOGFH "## endif: doproclist now : " . join (' ', @doproclist) . "\n";
+          print LOGFH "## endif: doproc: $doproc\n";
+          $iflevels =~ s/.\d+\s$//;
+        } else {
+          $doproc = 1;
+          $iflevels = '';
         }
+      }
 
-        if ($ininclude == 1 && $line =~ m#^endinclude$#o)
+      if ($doproc == 1) {
+        if ($line =~ m#^\s*output\s+([^\s]+)#o)
         {
-            print LOGFH "end include\n";
-            $ininclude = 0;
-            next;
-        }
-        elsif ($ininclude == 1)
-        {
-            $line =~ s,\\(.),$1,g;
-            $include .= $line . "\n";
-            next;
-        }
+            if ($inproc == 1) {
+              create_output (\%clist, \%config, $include);
+              $CONFH = 'none';
+              $include = '';
+            }
 
-        if ($inheaders && $line !~ m#^(hdr|sys)#o)
-        {
-            $inheaders = 0;
-        }
-
-        print LOGFH "#### ${linenumber}: ${line}\n";
-
-        if ($line =~ m#^output\s+([^\s]+)#o)
-        {
             print "output-file: $1\n";
-            print LOGFH "config file: $1\n";
-            $CONFH="../$1";
+            my $tconfh = $1;
+            if ($tconfh =~ m#^/#o) {
+              $CONFH = $tconfh;
+            } else {
+              $CONFH = "../$tconfh";
+            }
+            print LOGFH "config file: $CONFH\n";
+            $inproc = 1;
         }
-        elsif ($line =~ m#^(loadunit|standard)#o)
+        elsif ($line =~ m#^\s*option\-file\s+([^\s]+)#o)
+        {
+            print "option-file: $1\n";
+            my $tfile = $1;
+            if ($tfile =~ m#^/#o) {
+              $OPTIONFILE = $tfile;
+            } else {
+              $OPTIONFILE = "../$tfile";
+            }
+            print LOGFH "options file: $OPTIONFILE\n";
+        }
+        elsif ($line =~ m#^\s*standard#o)
+        {
+            check_standard (\%clist, \%config);
+        }
+        elsif ($line =~ m#^\s*loadunit#o)
         {
             ;
         }
-        elsif ($line =~ m#^setmntent_args#o)
+        elsif ($line =~ m#^\s*setmntent_args#o)
         {
             check_setmntent_args ('_setmntent_args', \%clist, \%config);
         }
-        elsif ($line =~ m#^statfs_args#o)
+        elsif ($line =~ m#^\s*statfs_args#o)
         {
             check_statfs_args ('_statfs_args', \%clist, \%config);
         }
-        elsif ($line =~ m#^include_time#o)
+        elsif ($line =~ m#^\s*include_conflict\s+([^\s]+)\s+([^\s]+)#o)
         {
-            check_include_time ('_include_time', \%clist, \%config);
+            my $i1 = $1;
+            my $i2 = $2;
+            check_include_conflict ('_inc_conflict', $i1, $i2, \%clist, \%config);
         }
-        elsif ($line =~ m#^include$#o)
+        elsif ($line =~ m#^\s*quotactl_pos$#o)
+        {
+            check_quotactl_pos ('_quotactl_pos', \%clist, \%config);
+        }
+        elsif ($line =~ m#^\s*include$#o)
         {
             print LOGFH "start include\n";
             $ininclude = 1;
         }
-        elsif ($line =~ m#^(hdr|sys)\s+([^\s]+)\s*(.*)#o)
+        elsif ($line =~ m#^\s*(hdr|sys)\s+([^\s]+)\s*(.*)#o)
         {
             my $typ = $1;
             my $hdr = $2;
             my $reqhdr = $3;
             my $nm = "_${typ}_";
             # create the standard header name for config.h
-            my @h = split (/\//, $hdr);
-            $nm .= join ('_', @h);
+            $nm .= $hdr;
+            $nm =~ s,/,_,go;
             $nm =~ s,\.h$,,o;
             $nm =~ s,:,_,go;
-            if ($typ eq 'sys')
-            {
-                $hdr = 'sys/' . $hdr;
+            if ($typ eq 'sys') {
+              $hdr = 'sys/' . $hdr;
             }
             $reqhdr =~ s/^\s*//o;
             $reqhdr =~ s/\s*$//o;
             my @oh = split (/\s+/, $reqhdr);
             check_header ($nm, $hdr, \%clist, \%config, { 'reqhdr' => \@oh, });
         }
-        elsif ($line =~ m#^const\s+([^\s]+)\s*(.*)#o)
+        elsif ($line =~ m#^\s*const\s+([^\s]+)\s*(.*)#o)
         {
             my $tnm = $1;
             my $reqhdr = $2;
@@ -1019,20 +1509,67 @@ create_config
             check_constant ($nm, $tnm, \%clist, \%config,
                 { 'reqhdr' => \@oh, });
         }
-        elsif ($line =~ m#^command\s+(.*)#o)
+        elsif ($line =~ m#^\s*command\s+(.*)#o)
         {
             my $cmd = $1;
             my $nm = "_command_" . $cmd;
             check_command ($nm, $cmd, \%clist, \%config);
         }
-        elsif ($line =~ m#^npt\s+([^\s]*)\s*(.*)#o)
+        elsif ($line =~ m#^\s*(if(not)?option)\s+([^\s]+)#o)
+        {
+            my $type = $1;
+            my $opt = $3;
+            my $nm = "_${type}_${opt}";
+            ++$ifstmtcount;
+            my $rc = check_ifoption ($ifstmtcount, $type, $nm, $opt, \%clist, \%config);
+            $iflevels .= "+$ifstmtcount ";
+            unshift @doproclist, $doproc;
+            $doproc = $rc;
+            print LOGFH "## ifoption: doproclist: " . join (' ', @doproclist) . "\n";
+            print LOGFH "## ifoption: doproc: $doproc\n";
+        }
+        elsif ($line =~ m#^\s*if\s+([^\s]+)\s+(.*)#o)
+        {
+            my $iflabel = $1;
+            my $ifline = $2;
+            ++$ifstmtcount;
+            print LOGFH "## if: label: $iflabel count $ifstmtcount line: $ifline\n";
+            my $rc = check_if ($iflabel, $ifstmtcount, $ifline, \%clist, \%config);
+            $iflevels .= "+$ifstmtcount ";
+            unshift @doproclist, $doproc;
+            $doproc = $rc;
+            print LOGFH "## if: doproclist: " . join (' ', @doproclist) . "\n";
+            print LOGFH "## if: doproc: $doproc\n";
+        }
+        elsif ($line =~ m#^\s*(endif|else)#o)
+        {
+            ;
+        }
+        elsif ($line =~ m#^\s*(set(int|str)?)\s+([^\s]+)\s*(.*)#o)
+        {
+            my $type = $1;
+            my $nm = $3;
+            if ($type eq 'setint' || $type eq 'setstr') {
+              $nm = "_${type}_$3";
+            }
+            my $val = $4;
+            check_set ($nm, $type, $val, \%clist, \%config);
+        }
+        elsif ($line =~ m#^\s*option\s+([^\s]+)\s*(.*)#o)
+        {
+            my $nm = "_opt_$1";
+            my $onm = $1;
+            my $def = $2;
+            check_option ($nm, $onm, $def, \%clist, \%config);
+        }
+        elsif ($line =~ m#^\s*npt\s+([^\s]*)\s*(.*)#o)
         {
             my $func = $1;
             my $req = $2;
             my $nm = "_npt_" . $func;
             check_npt ($nm, $func, $req, \%clist, \%config);
         }
-        elsif ($line =~ m#^key\s+(.*)#o)
+        elsif ($line =~ m#^\s*key\s+(.*)#o)
         {
             my $tnm = $1;
             my $nm = "_key_" . $tnm;
@@ -1042,7 +1579,7 @@ create_config
                 check_keyword ($nm, $tnm, \%clist, \%config);
             }
         }
-        elsif ($line =~ m#^class\s+([^\s]+)\s*(.*)?#o)
+        elsif ($line =~ m#^\s*class\s+([^\s]+)\s*(.*)?#o)
         {
             my $class = $1;
             my $libs = $2 || '';
@@ -1055,17 +1592,28 @@ create_config
                        { 'otherlibs' => $libs, });
             }
         }
-        elsif ($line =~ m#^typ\s+(.*)#o)
+        elsif ($line =~ m#^\s*typ\s+(.*)#o)
         {
             my $tnm = $1;
             my $nm = "_typ_" . $tnm;
+            $nm =~ s, ,_,go;
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
                 check_type ($nm, $tnm, \%clist, \%config);
             }
         }
-        elsif ($line =~ m#^lib\s+([^\s]+)\s*(.*)?#o)
+        elsif ($line =~ m#^\s*define\s+(.*)#o)
+        {
+            my $tnm = $1;
+            my $nm = "_define_" . $tnm;
+            if (! defined ($config{$nm}) ||
+                $config{$nm} eq '0')
+            {
+                check_defined ($nm, $tnm, \%clist, \%config);
+            }
+        }
+        elsif ($line =~ m#^\s*lib\s+([^\s]+)\s*(.*)?#o)
         {
             my $func = $1;
             my $libs = $2 || '';
@@ -1077,7 +1625,7 @@ create_config
                        { 'otherlibs' => $libs, });
             }
         }
-        elsif ($line =~ m#^dcl\s+([^\s]*)\s+(.*)#o)
+        elsif ($line =~ m#^\s*dcl\s+([^\s]*)\s+(.*)#o)
         {
             my $type = $1;
             my $var = $2;
@@ -1095,18 +1643,19 @@ create_config
                 }
             }
         }
-        elsif ($line =~ m#^member\s+(.*)\s+(.*)#o)
+        elsif ($line =~ m#^\s*member\s+(.*)\s+([^\s]+)#o)
         {
             my $struct = $1;
             my $member = $2;
-            my $nm = "_mem_" . $member . '_' . $struct;
+            my $nm = "_mem_" . $struct . '_' . $member;
+            $nm =~ s/ /_/go;
             if (! defined ($config{$nm}) ||
                 $config{$nm} eq '0')
             {
                 check_member ($nm, $struct, $member, \%clist, \%config);
             }
         }
-        elsif ($line =~ m#^size\s+(.*)#o)
+        elsif ($line =~ m#^\s*size\s+(.*)#o)
         {
             my $typ = $1;
             $typ =~ s/\s*$//o;
@@ -1123,83 +1672,31 @@ create_config
             print LOGFH "unknown command: $line\n";
             print STDOUT "unknown command: $line\n";
         }
-    }
-
-    open (CCOFH, ">$CONFH");
-    print CCOFH <<'_HERE_';
-#ifndef __INC_CONFIG_H
-#define __INC_CONFIG_H 1
-
-_HERE_
-
-    foreach my $val (@{$clist{'list'}})
-    {
-      my $tval = 0;
-      if ($config{$val} ne "0")
-      {
-          $tval = 1;
-      }
-      if ($val =~ m#^(_hdr|_sys|_command)#o)
-      {
-        print CCOFH "#define $val $tval\n";
-      }
-      else
-      {
-        print CCOFH "#define $val $config{$val}\n";
       }
     }
 
-    # standard tail -- always needed; non specific
-    print CCOFH <<'_HERE_';
-
-#if ! _key_void
-# define void int
-#endif
-#if ! _key_const
-# define const
-#endif
-
-#ifndef _
-# if _proto_stdc
-#  define _(args) args
-# else
-#  define _(args) ()
-# endif
-#endif
-
-_HERE_
-
-    print CCOFH $include;
-
-    print CCOFH <<'_HERE_';
-
-#endif /* __INC_CONFIG_H */
-_HERE_
-    close CCOFH;
+    savecache (\%clist, \%config);
+    create_output (\%clist, \%config, $include);
 
+    # This is here for os/2 and other systems w/no usable bourne shell.
+    # it is not up to date, as it will output libraries that are not
+    # needed by the final configuration.
     open (RLIBFH, ">$REQLIB");
-
     my $r_list = $config{'reqlibs_list'};
     print RLIBFH join (' ', @{$r_list}) . "\n";
     close RLIBFH;
-
-    savecache (\%clist, \%config);
 }
 
 sub
 usage
 {
-  print STDOUT "Usage: $0 [-c <cache-file>] [-v <vars-file>] \n";
-  print STDOUT "       [-l <log-file>] [-t <tmp-dir>] [-r <reqlib-file>]\n";
-  print STDOUT "       [-C] <config-file>\n";
+  print STDOUT "Usage: $0 [-c <cache-file>] [-L <log-file>]\n";
+  print STDOUT "       [-o <option-file>] [-C] <config-file>\n";
   print STDOUT "  -C : clear cache-file\n";
-  print STDOUT "<tmp-dir> must not exist.\n";
   print STDOUT "defaults:\n";
   print STDOUT "  <cache-file> : mkconfig.cache\n";
-  print STDOUT "  <vars-file>  : mkconfig.vars\n";
   print STDOUT "  <log-file>   : mkconfig.log\n";
-  print STDOUT "  <tmp-dir>    : _tmp_mkconfig\n";
-  print STDOUT "  <reqlib-file>: reqlibs.txt\n";
+  print STDOUT "  <option-file>: options.dat\n";
 }
 
 # main
@@ -1218,16 +1715,16 @@ while ($#ARGV > 0)
       $CACHEFILE = $ARGV[0];
       shift @ARGV;
   }
-  if ($ARGV[0] eq "-l")
+  if ($ARGV[0] eq "-L")
   {
       shift @ARGV;
       $LOG = $ARGV[0];
       shift @ARGV;
   }
-  if ($ARGV[0] eq "-t")
+  if ($ARGV[0] eq "-o")
   {
       shift @ARGV;
-      $TMP = $ARGV[0];
+      $OPTIONFILE = $ARGV[0];
       shift @ARGV;
   }
 }
@@ -1248,6 +1745,7 @@ $LOG = "../$LOG";
 $REQLIB = "../$REQLIB";
 $CACHEFILE = "../$CACHEFILE";
 $VARSFILE = "../$VARSFILE";
+$OPTIONFILE = "../$OPTIONFILE";
 
 if (-d $TMP) { system ("rm -rf $TMP"); }
 mkdir $TMP, 0777;
@@ -1262,13 +1760,14 @@ if ($clearcache)
 print STDOUT "$0 using $configfile\n";
 unlink $LOG;
 open (LOGFH, ">>$LOG");
-$ENV{'CFLAGS'} = $ENV{'CFLAGS'} . ' ' . $ENV{'CINCLUDES'};
+$ENV{'CFLAGS'} = $ENV{'CFLAGS'};
 print LOGFH "CC: $ENV{'CC'}\n";
 print LOGFH "CFLAGS: $ENV{'CFLAGS'}\n";
+print LOGFH "CPPFLAGS: $ENV{'CPPFLAGS'}\n";
 print LOGFH "LDFLAGS: $ENV{'LDFLAGS'}\n";
 print LOGFH "LIBS: $ENV{'LIBS'}\n";
 
-create_config $configfile;
+main_process $configfile;
 
 close LOGFH;
 
