@@ -3,19 +3,22 @@
 #  Copyright 2010 Brad Lanam Walnut Creek, CA USA
 #
 
-echo ${EN} "rpmbuild${EC}" >&3
-
-cd $RUNTOPDIR
-. ./mkconfig.cache
-
-if [ "${di_c__command_rpmbuild}" = "0" ];then
-  echo ${EN} " skipped${EC}" >&3
+if [ "$1" = "-d" ]; then
+  echo ${EN} " rpmbuild${EC}"
   exit 0
 fi
 
-rvers=`rpmbuild --version | tr -cd '0-9'`
+cd $_MKCONFIG_RUNTOPDIR
+. ./mkconfig.cache
+
+if [ "${di_c__command_rpmbuild}" = "0" ];then
+  echo ${EN} " skipped${EC}" >&5
+  exit 0
+fi
+
+rvers=`rpmbuild --version | tr -cd '0-9' | sed 's/^\(...\).*/\1/'`
 if [ $rvers -lt 470 ]; then
-  echo ${EN} " old version skipped${EC}" >&3
+  echo ${EN} " old version skipped${EC}" >&5
   exit 0
 fi
 
@@ -25,5 +28,8 @@ grc=0
 make -e DI_DIR="." DI_VERSION=${DI_VERSION} testrpmbuild
 rc=$?
 if [ $rc -ne 0 ]; then grc=$rc; fi
+# leave a copy there...realclean will get them...
+cp mkconfig.log mkconfig.cache mkconfig*.vars di.env mkconfig.reqlibs \
+    $_MKCONFIG_TSTRUNTMPDIR
 
 exit $grc
